
From: Ian Kent <raven@themaw.net>

For browsable autofs maps, a mount request that arrives at the same time an
expire is happening can fail to perform the needed mount.

This happens becuase the directory exists and so the revalidate succeeds when
we need it to fail so that lookup is called on the same dentry to do the
mount.  Instead lookup is called on the next path component which should be
whithin the mount, but the parent isn't mounted.  

The solution is to allow the revalidate to continue and perform the mount as
no directory creation (at mount time) is needed for browsable mount entries.

Signed-off-by: Ian Kent <raven@themaw.net>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 fs/autofs4/root.c |    9 ++++++++-
 1 files changed, 8 insertions(+), 1 deletion(-)

diff -puN fs/autofs4/root.c~autofs4-bad-lookup-fix fs/autofs4/root.c
--- 25/fs/autofs4/root.c~autofs4-bad-lookup-fix	2005-06-05 23:34:55.000000000 -0700
+++ 25-akpm/fs/autofs4/root.c	2005-06-05 23:34:55.000000000 -0700
@@ -306,7 +306,14 @@ static int try_to_fill_dentry(struct den
 		
 		DPRINTK("expire done status=%d", status);
 		
-		return 0;
+		/*
+		 * If the directory still exists the mount request must
+		 * continue otherwise it can't be followed at the right
+		 * time during the walk.
+		 */
+		status = d_invalidate(dentry);
+		if (status != -EBUSY)
+			return 0;
 	}
 
 	DPRINTK("dentry=%p %.*s ino=%p",
_
